# Include this concern in a model if the following is the case:
#
# - The model that has a uniqueness validator
# - Another model contains a has_many relation pointing to the model
# - The model is used in a nested form for multi update/insert.
#
# The inclusion will fix the following bug/unwanted behavior:
#
# when a nested form with multiple records (having unique constraints) is saved,
# the complete validations run on the records first, an after that they are persisted.
# This has the effect, that no duplicates are found, as the unique validator
# simply does a query to detect duplicates.
#
# This is only a problem of usability, as there should always be a db level constraint,
# but still a problem, as an exception is raised and no form error is shown.
#
# The only drawback with this concern is, that the uniqueness validation in fact runs twice,
# once normally, when all validations run, and later in a before save hook.
module UniqueValidatorsNestedFormsConcern
  extend ActiveSupport::Concern
  included do

    before_save do
      valid = true
      self.class.validators.each do |validator|
        if validator.kind == :uniqueness
          validator.validate(self)
          validator.attributes.each do | attr |
            valid &&= !self.errors.include?(attr)
          end
        end
      end
      valid
    end

  end





end