module Portfolio
  module Concerns
    module DiagnosticResultConcern
        extend ActiveSupport::Concern

        # included can be:
        # ::Portfolio::ItemClassDiagnostic
        # ::Portfolio::ItemDiagnostic
        # ::Portfolio::ItemDiagnosticMeasurement
        included do
          has_many :informal_results, as: :diagnostic_result_acceptor,
                   foreign_key: :diagnostic_result_acceptor_id,
                   class_name: '::Portfolio::DiagnosticInformalResult',
                   dependent: :destroy # refs #192

          # not all models including the module allow all types of involved persons.
          # A class can define:
          #
          # include ::Portfolio::Concerns::DiagnosticResultConcern
          # self.allowed_involvements[:teacher] = false
          # self.allowed_involvements[:subject] = false
          #
          # This excludes teachers and the portfolio subject (client or client_group) from the available persons.
          #
          # Here, all available person types are listed and also allowed by default, including classes should opt out.
          mattr_accessor :allowed_involvements do
            { teacher: true, guardian: true, supporter: true, person: true, subject: true }
          end


          # when accepting formal results, the including model must have a belongs_to :diagnostic asscociation
          # which holds an instance of ::Diagnostic
          #
          # If that is ensured, define like this:
          #
          # include ::Portfolio::Concerns::DiagnosticResultConcern
          # self.accepts_formal_results = true
          mattr_accessor :accepts_formal_results do
            false
          end


        end

        # this is the name of the included, as shown in result views
        def name
          case self
            when ::Portfolio::ItemClassDiagnostic
              "#{diagnostic.title}"
            when ::Portfolio::ItemDiagnostic
              "#{diagnostic.title}"
            when ::Portfolio::ItemDiagnosticMeasurement
              "Freie Diagnostik"
          end
        end


        def result_allows_involvement_of?( involvement_type )
          self.allowed_involvements[involvement_type]
        end
    end
  end
end