module Portfolio
  class DiagnosticInformalResult < ::ActiveRecord::Base
    self.table_name = 'portfolio_item_diagnostic_informal_results'
    self.inheritance_column = nil

    has_many :results,
             foreign_key: :portfolio_item_diagnostic_informal_result_id,
             class_name: '::Portfolio::ItemDiagnosticResult',
             inverse_of: :portfolio_diagnostic_informal_result,
             dependent: :destroy # refs #192
    accepts_nested_attributes_for :results

    # this points to through ::Portfolio::Concerns::DiagnosticResultConcern to any model that can include results
    # the presence is mandatory
    belongs_to :diagnostic_result_acceptor, polymorphic: true
    validates_presence_of :diagnostic_result_acceptor
    delegate :content, to: :diagnostic_result_acceptor

    # when a new diagnostic_result_acceptor is set,
    # it marks for destruction and builds new results (keeping existing ones) until the set of results
    # matches the new characteristics and norm scales
    def build_formal_results
      if diagnostic_result_acceptor.respond_to?(:accepts_formal_results) && diagnostic_result_acceptor.accepts_formal_results
        _r = results.all.to_a
        _r.each(&:mark_for_destruction)
        # here we can assume that the acceptor has a relation named 'diagnostic' of type ::Diagnostic
        diagnostic_result_acceptor.diagnostic.characteristics.each do |characteristic|
          diagnostic_result_acceptor.diagnostic.norm_scales.each do |norm|
            if existing = _r.find {|r| r.characteristic==characteristic &&  r.norm_scale==norm }
              existing.reload # clears mark for destruction
            else
              _r << results.build({characteristic: characteristic, norm_scale: norm, result: nil })
            end
          end
          if existing =  _r.find {|r| r.characteristic==characteristic && r.norm_scale==nil }
            existing.reload # clears mark for destruction
          else
            _r << results.build({characteristic: characteristic, norm_scale: nil, result: nil }) # nil is 'Rohwerte'
          end
        end
      end
    end


    has_many :portfolio_diagnostic_informal_result_teachers,
             foreign_key: :portfolio_item_diagnostic_informal_result_id,
             class_name: '::Portfolio::DiagnosticInformalResultTeacher',
             inverse_of: :diagnostic_informal_result,
             dependent: :destroy # refs #192

    has_many :involved_teachers,
             through: :portfolio_diagnostic_informal_result_teachers,
             class_name: '::Teacher',
             source: :teacher, validate: false

    has_many :portfolio_diagnostic_informal_result_guardians,
             foreign_key: :portfolio_item_diagnostic_informal_result_id,
             class_name: '::Portfolio::DiagnosticInformalResultGuardian',
             inverse_of: :diagnostic_informal_result,
             dependent: :destroy # refs #192

    has_many :involved_guardians,
             through: :portfolio_diagnostic_informal_result_guardians,
             class_name: '::Guardian',
             source: :guardian, validate: false

    has_many :portfolio_diagnostic_informal_result_supporters,
             foreign_key: :portfolio_item_diagnostic_informal_result_id,
             class_name: '::Portfolio::DiagnosticInformalResultSupporter',
             inverse_of: :diagnostic_informal_result,
             dependent: :destroy # refs #192

    has_many :involved_supporters,
             through: :portfolio_diagnostic_informal_result_supporters,
             class_name: '::Supporter',
             source: :supporter, validate: false

    has_many :involved_people,
             foreign_key: :portfolio_item_diagnostic_informal_result_id,
             class_name: '::Portfolio::DiagnosticInformalResultPerson',
             inverse_of: :diagnostic_informal_result,
             dependent: :destroy # refs #192
    accepts_nested_attributes_for :involved_people, :allow_destroy => true, reject_if: :all_blank

    # these are just documents that are attached to the result
    has_many :documents,
        foreign_key: :portfolio_item_diagnostic_informal_result_id,
        class_name: '::Portfolio::DiagnosticInformalResultDocument',
        dependent: :destroy # refs #192
    accepts_nested_attributes_for :documents, :allow_destroy => true, reject_if: :all_blank

    # get a description why this item cannot be closed. This is useful when can_be_closed? returned false
    #
    # @return [String] reason why cannot be closed
    def cannot_be_deleted_reason
      return "Der zugehörige Eintrag ist schon abgeschlossen"
    end



    # check if an item can be deleted
    #
    # @return [Boolean] true when the item can be deleted
    def can_be_deleted?
      ! diagnostic_result_acceptor.portfolio_item.closed?
    end

  end
end


