class Teacher < ActiveRecord::Base

  #### Lehrkräfte: alle Personen, die direkt mit Förder-/Erziehungsmaßnahmen betraut sind: z.B. Lehrer, Sonderpädagogen

  ENUM_GENDER_MALE = 0
  ENUM_GENDER_FEMALE = 1

  include Identifiable
  include UserMessageConcern

  #--- Behavior
  #acts_as_paranoid

  #--- Associations
  has_many :career_items

  has_many :client_groups
  # TODO auch die client_groups denen er mal zugeordnet war? (client_group_histories)
  has_one :current_client_group, ->{ active }, class_name: 'ClientGroup'

  # Verlauf der Klassenleitung
  has_many :client_group_histories

  belongs_to :teacher_category

  has_many :teacher_institution_relations, dependent: :destroy, inverse_of: :teacher
  has_many :institutions, through: :teacher_institution_relations

  has_many :portfolio_item_involved_teachers,
           class_name: '::Portfolio::ItemInvolvedTeacher',
           foreign_key: :teacher_id,
           inverse_of: :teacher

  has_many :portfolio_item_involvements,
           class_name: '::Portfolio::Item',
           through: :portfolio_item_involved_teachers,
           source: :portfolio_item

  has_many :portfolio_item_diagnostic_informal_result_teachers,
           class_name: '::Portfolio::DiagnosticInformalResultTeacher',
           foreign_key: :teacher_id,
           inverse_of: :teacher

  has_many :portfolio_item_diagnostic_informal_results,
           class_name: '::Portfolio::DiagnosticInformalResult',
           through: :portfolio_item_diagnostic_informal_result_teachers,
           source: :diagnostic_informal_result

  has_many :portfolio_comments,
           class_name: '::Portfolio::Comment',
           foreign_key: :teacher_id,
           inverse_of: :teacher


  has_many :diagnostic_borrows
  has_many :advancement_plan_borrows

  accepts_nested_attributes_for :teacher_institution_relations, :allow_destroy => true, reject_if: :all_blank

  # get the latest portfolio items for a teacher, the latest item from every distinct portfolio
  #
  # @return [ActiveRecordRelation<Portfolio::Item>] collection of portfolio items
  def latest_portfolio_items
    latest_items = {}
    associated_portfolio_items.select(:id,:portfolio_id).order('start asc,created_at asc').each { |item| latest_items[item.portfolio_id] = item.id }
    ::Portfolio::Item.where(id: latest_items.values)
  end

  def associated_client_groups
    ::ClientGroup.active.where('client_groups.teacher_id = ? OR client_groups.id IN (?)',id,associated_clients.pluck(:client_group_id))
  end

  def associated_portfolios
    portfolio_ids = ::Portfolio::Portfolio.find_by_sql(['
          select distinct p.id from portfolios p
          inner join clients c on c.id=p.subject_id and p.subject_type=?
          where c.teacher_id=?
          UNION
          select distinct p.id from portfolios p
          inner join client_groups cg on cg.id=p.subject_id and p.subject_type=?
          inner join clients c on c.client_group_id=cg.id
          where c.teacher_id=?
          UNION
          select distinct p.id from portfolios p
          inner join clients c on c.id=p.subject_id and p.subject_type=?
          inner join client_groups cg on cg.id=c.client_group_id
          where cg.teacher_id=?
          UNION
          select distinct p.id from portfolios p
          inner join client_groups cg on cg.id=p.subject_id and p.subject_type=?
          where cg.teacher_id=?', 'Client', id, 'ClientGroup', id, 'Client', id,'ClientGroup', id ])
    ::Portfolio::Portfolio.where(id:  portfolio_ids )
  end

  # get all portfolio items where the teacher is associated with the portfolio item
  #
  # That means, portfolio items are returned that match one of the following:
  #
  # 1. items where teacher involved in portfolio item
  # 2. items where teacher involved in evaluation of portfolio item
  # 3. items where teacher is author of portfolio item
  # 4. clients where teacher is set for special education (teacher_id foreign key of client)
  # 5. clients where teacher is set as class teacher (client_group teacher)
  #
  # @return [ActiveRecordRelation<Portfolio::Item>] collection of portfolio items
  def associated_portfolio_items
    ::Portfolio::Item.where(portfolio_id:  associated_portfolios.ids )
  end

  def associated_clients
    # 1. clients where teacher involved in portfolio
    # 2. clients where teacher involved in evaluation of portfolio
    # 3. clients where teacher is set for special education (teacher_id foreign key of client)
    # 4. clients where teacher is set as class teacher (client_group teacher)
    # 5. clients where teacher is author of portfolio item
    client_ids = ::Client.find_by_sql(['select distinct c.id from clients c
      inner join portfolios p on p.subject_id=c.id and p.subject_type=?
      inner join portfolio_items pi on pi.portfolio_id=p.id
      inner join portfolio_item_involved_teachers piit on piit.portfolio_item_id=pi.id
      inner join teachers t on t.id=piit.teacher_id
      where piit.teacher_id=?
      UNION
      select distinct c.id from clients c
      inner join client_groups cg on c.client_group_id=cg.id
      inner join portfolios p on p.subject_id=cg.id and p.subject_type=?
      inner join portfolio_items pi on pi.portfolio_id=p.id
      inner join portfolio_item_involved_teachers piit on piit.portfolio_item_id=pi.id
      inner join teachers t on t.id=piit.teacher_id
      where piit.teacher_id=?
      UNION
      select distinct c.id from clients c
      inner join portfolios p on p.subject_id=c.id and p.subject_type=?
      inner join portfolio_items pi on pi.portfolio_id=p.id
      inner join portfolio_item_targets pit on pit.portfolio_item_id= pi.id
      inner join portfolio_item_evaluations pie on pie.portfolio_item_target_id= pit.id
      inner join evaluation_involved_teachers eit on eit.portfolio_item_evaluation_id=pie.id
      where eit.teacher_id=?
      UNION
      select distinct c.id from clients c
      inner join client_groups cg on c.client_group_id=cg.id
      inner join portfolios p on p.subject_id=cg.id and p.subject_type=?
      inner join portfolio_items pi on pi.portfolio_id=p.id
      inner join portfolio_item_targets pit on pit.portfolio_item_id= pi.id
      inner join portfolio_item_evaluations pie on pie.portfolio_item_target_id= pit.id
      inner join evaluation_involved_teachers eit on eit.portfolio_item_evaluation_id=pie.id
      where eit.teacher_id=?
      UNION
      select distinct c.id from clients c
      where c.teacher_id=?
      UNION
      select distinct c.id from clients c
      inner join client_groups cg on cg.id=c.client_group_id
      where cg.teacher_id=?
      UNION
      select distinct c.id from clients c
      inner join portfolios p on p.subject_id=c.id and p.subject_type=?
      inner join portfolio_items pi on pi.portfolio_id=p.id
      where pi.author_type=? and pi.author_id=?
      UNION
      select distinct c.id from clients c
      inner join client_groups cg on c.client_group_id=cg.id
      inner join portfolios p on p.subject_id=cg.id and p.subject_type=?
      inner join portfolio_items pi on pi.portfolio_id=p.id
      where pi.author_type=? and pi.author_id=?', 'Client',id,'ClientGroup',id,'Client',id,'ClientGroup',id,id,id,'Client','Teacher',id,'ClientGroup','Teacher',id])
    ::Client.where(id: client_ids)
  end

  #--- Attributes
  enum gender: { "männlich" => ENUM_GENDER_MALE, "weiblich" => ENUM_GENDER_FEMALE }

  #--- General Validations
  # validates :institutions,  :presence => true
  validates :teacher_category,  :presence => true
  validates :first_name, :last_name, :gender, :presence => true
  validates :email, email: {}
  validates :phone, :mobile, :fax, phony_plausible: {normalized_country_code: 'DE'}

  validates :teacher_institution_relations, strict_length: { minimum: 1 }

  #--- Conditional Validations
  validate do
    res = true
    if current_client_group && institutions
      res = false
      institutions.each do |i|
        if current_client_group.institution == i
          res = true
          break
        end
      end
      unless res
        errors.add(:base, "Es muss mindestens die Schule ausgewählt werden, an der eine Tätigkeit als KlassenlehrerIn durchgeführt wird: " + current_client_group.institution.name + ".")
      end
    end
    res
  end



  before_destroy :is_destructable?


  # teachers category at a given institution
  def teacher_institution_category(institution_id)
    TeacherInstitutionCategory.joins(:teacher_institution_relations)
        .where('teacher_institution_relations.teacher_id=? and teacher_institution_relations.institution_id=?',id,institution_id)
        .first
  end

  # def system_notifications
  #   client_ids = associated_clients.ids
  #   portfolio_item_ids = associated_portfolio_items.ids
  #   client_group_ids = associated_client_groups.ids
  #   portfolio_ids = associated_portfolios.ids
  #   institution_ids = TeacherInstitutionRelation.where(teacher_id: id).pluck(:institution_id)
  #   client_condition = '( notificationable_type=? AND notificationable_id IN (?) )'
  #   portfolio_item_condition = '( notificationable_type=? AND notificationable_id IN (?) )'
  #   semester_condition = '( notificationable_type=? )'
  #   client_group_condition = '( notificationable_type=? AND notificationable_id IN (?) )'
  #   client_duplicate_requests_condition = '( notificationable_type IN (?) AND institution_id IN (?)  )'
  #   portfolio_condition = '( notificationable_type IN (?) AND notificationable_id IN (?)  )'
  #   ::Notification.only_showable.where("#{client_condition} OR #{portfolio_item_condition} OR #{semester_condition} OR #{client_group_condition} OR #{client_duplicate_requests_condition} OR #{portfolio_condition}",
  #         'Client', client_ids, 'Portfolio::Item', portfolio_item_ids, 'Semester', 'ClientGroup', client_group_ids,
  #         'ClientDuplicateRequest',  institution_ids, 'Portfolio::Portfolio', portfolio_ids)
  # end

  def full_name(client_or_client_group = nil)
    function = []
    if client_or_client_group.is_a? ::Client
      client = client_or_client_group
      institution_category = teacher_institution_category(client.institution_id)
      function << "Leitung #{client.institution.name}" if institution_category && institution_category.is_management == true
      function << "Klassenlehrer #{client.client_group.name}" if client.client_group && client.client_group.teacher == self
      function << 'betreuender Sonderpädagoge' if client.teacher == self
    end
    if client_or_client_group.is_a? ::ClientGroup
      client_group = client_or_client_group
      institution_category = teacher_institution_category(client_group.institution_id)
      function << "Leitung #{client_group.institution.name}" if institution_category && institution_category.is_management == true
      function << "Klassenlehrer #{client_group.name}" if client_group && client_group.teacher == self
      function << 'Sonderpädagoge' if self.teacher_category.is_special_ed?
    end
    if function.present?
      return  first_name + " " + last_name + " (#{function.join(', ')}) "
    else
      return  first_name + " " + last_name
    end

  end


  def is_destructable?
    true
  end



end
